/*
 * board_mvblx.c (MATRIX VISION GmbH mvBlueLYNX-X)
 *
 * uses bits from board-rx51-camera.c
 *
 * based on....
 *
 * board-overo.c (Gumstix Overo)
 *
 * Initial code: Steve Sakoman <steve@sakoman.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#define USE_SPIDEV_DEBUG 0

#include <linux/clk.h>
#include <linux/delay.h>
#include <linux/err.h>
#include <linux/init.h>
#include <linux/io.h>
#include <linux/kernel.h>
#include <linux/platform_device.h>
#include <linux/input/matrix_keypad.h>
#include <linux/i2c/twl.h>
#include <linux/i2c/at24.h>
#include <linux/regulator/machine.h>
#include <linux/regulator/fixed.h>
#include <linux/spi/spi.h>
#include <linux/w1-gpio.h>
#include <linux/leds.h>
#include <linux/crc32.h>

#include <linux/mmc/host.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>

#include <plat/board.h>
#include <plat/common.h>
#include <plat/display.h>
#include <plat/panel-generic-dpi.h>
#include <mach/gpio.h>
#include <plat/gpmc.h>
#include <mach/hardware.h>
#include <plat/mcspi.h>
#include <plat/mux.h>
#include <plat/usb.h>
#include <plat/clock.h>

#if defined(CONFIG_MEDIA_CONTROLLER)
#include "../../../drivers/media/video/omap3isp/isp.h"
#endif

#include "mux.h"
#include "sdram-micron-mt46h32m32lf-6.h"
#include "hsmmc.h"
#include "timer-gp.h"

#include "devices.h"

#define MVBLX_GPIO_USBH_NRESET	26
#define MVBLX_GIPO_ONE_WIRE		158



/*
 * mvBlueLYNX-X revision
 * Run time detection of mvBlueLYNX-X revision is done by reading GPIO.
 *	Prototype	= no GPIO pins used
 *	Rev. 1		= GPIO150, GPIO24: 1 1
 *	Rev. 2		= GPIO150, GPIO24: 1 0
 *	Rev. 3		= GPIO150, GPIO24: 0 1
 *	Rev. 4		= GPIO150, GPIO24: 0 0
 */

#define MVBLX_GPIO_REVID_0 24
#define MVBLX_GPIO_REVID_1 150

enum {
	MVBLX_BOARD_UNKNOWN = 0,
	MVBLX_BOARD_1,
	MVBLX_BOARD_2,
	MVBLX_BOARD_3,
	MVBLX_BOARD_4,
};

static u8 mvblx_version;

static u8 mvblx_get_rev(void)
{
	return mvblx_version;
}

static void __init mvblx_init_rev(void)
{
	int ret;
	u16 mvblx_rev = 0;

	omap_mux_init_gpio(MVBLX_GPIO_REVID_0, OMAP_PIN_INPUT_PULLUP);
	omap_mux_init_gpio(MVBLX_GPIO_REVID_1, OMAP_PIN_INPUT_PULLUP);

	ret = gpio_request(MVBLX_GPIO_REVID_0, "rev_id_0");
	if (ret < 0)
		goto fail0;

	ret = gpio_request(MVBLX_GPIO_REVID_1, "rev_id_1");
	if (ret < 0)
		goto fail1;

	gpio_direction_input(MVBLX_GPIO_REVID_0);
	gpio_direction_input(MVBLX_GPIO_REVID_1);

	mvblx_rev = gpio_get_value(MVBLX_GPIO_REVID_0) | (gpio_get_value(MVBLX_GPIO_REVID_1) << 1);

	gpio_free(MVBLX_GPIO_REVID_0);
	gpio_free(MVBLX_GPIO_REVID_1);

	switch (mvblx_rev) {
	case 0x00:
		printk(KERN_INFO "mvBlueLYNX-X Rev: 4\n");
		mvblx_version = MVBLX_BOARD_4;;
		break;
	case 0x01:
		printk(KERN_INFO "mvBlueLYNX-X Rev: 3\n");
		mvblx_version = MVBLX_BOARD_3;
		break;
	case 0x02:
		printk(KERN_INFO "mvBlueLYNX-X Rev: 2\n");
		mvblx_version = MVBLX_BOARD_2;
		break;
	case 0x03:
		printk(KERN_INFO "mvBlueLYNX-X Rev: 1\n");
		mvblx_version = MVBLX_BOARD_1;
		break;
	default:
		printk(KERN_INFO "mvBlueLYNX-X Rev: unknown\n");
		mvblx_version = MVBLX_BOARD_UNKNOWN;
	}

	return;

fail1:
	gpio_free(MVBLX_GPIO_REVID_0);
fail0:
	printk(KERN_INFO "Unable to get revision detection GPIO pins\n");
	printk(KERN_INFO "mvBlueLYNX-X Rev: unknown\n");
	mvblx_version = MVBLX_BOARD_UNKNOWN;

	return;
}


#if defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE)

#include <linux/smsc911x.h>

static struct smsc911x_platform_config mvblx_smsc911x_config = {
	.phy_interface  = PHY_INTERFACE_MODE_MII,
	.irq_polarity   = SMSC911X_IRQ_POLARITY_ACTIVE_LOW,
	.irq_type       = SMSC911X_IRQ_TYPE_OPEN_DRAIN,
	.flags          = SMSC911X_USE_32BIT,
	/* .mac			will be read from the EEPROM during i2c initialization */
};

#define MVBLX_SMSC911X_CS	0
#define MVBLX_SMSC911X_GPIO	137
#define MVBLX_SMSC911X_RESET	139

static struct resource mvblx_smsc911x_resources[] = {
	{
		.name	= "smsc911x-memory",
		.flags	= IORESOURCE_MEM,
	},
	{
		.flags	= IORESOURCE_IRQ | IORESOURCE_IRQ_LOWLEVEL,
	},
};

static struct platform_device mvblx_smsc911x_device = {
	.name		= "smsc911x",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(mvblx_smsc911x_resources),
	.resource	= mvblx_smsc911x_resources,
	.dev		= {
		.platform_data = &mvblx_smsc911x_config,
	},
};

static struct platform_device *smsc911x_devices[] = {
	&mvblx_smsc911x_device,
};

static inline void __init mvblx_init_smsc911x(void)
{
	unsigned long cs_mem_base;

	/* set up smsc911x chip */
	if (gpmc_cs_request(MVBLX_SMSC911X_CS, SZ_16M, &cs_mem_base) < 0) {
		printk(KERN_ERR "Failed request for GPMC mem for smsc911x\n");
		return;
	}

	mvblx_smsc911x_resources[0].start = cs_mem_base + 0x0;
	mvblx_smsc911x_resources[0].end   = cs_mem_base + 0xff;

	if ((gpio_request(MVBLX_SMSC911X_GPIO, "SMSC911X IRQ") == 0) &&
	    (gpio_direction_input(MVBLX_SMSC911X_GPIO) == 0)) {
		gpio_export(MVBLX_SMSC911X_GPIO, 0);
	} else {
		printk(KERN_ERR "could not obtain gpio for SMSC911X IRQ\n");
		return;
	}

	mvblx_smsc911x_resources[1].start = OMAP_GPIO_IRQ(MVBLX_SMSC911X_GPIO);
	mvblx_smsc911x_resources[1].end	  = 0;

	platform_add_devices(smsc911x_devices, ARRAY_SIZE(smsc911x_devices));
}

#endif /* SMSC911X */

/* DSS */

static int mvblx_enable_dvi(struct omap_dss_device *dssdev)
{
	return 0;
}

static void mvblx_disable_dvi(struct omap_dss_device *dssdev)
{
}

static struct panel_generic_dpi_data dvi_panel = {
	.name = "monitor",
	.platform_enable = mvblx_enable_dvi,
	.platform_disable = mvblx_disable_dvi,
};

static struct omap_dss_device mvblx_dvi_device = {
	.type = OMAP_DISPLAY_TYPE_DPI,
	.name = "dvi",
	.data = &dvi_panel,
	.driver_name = "generic_dpi_panel",
	.phy.dpi.data_lines = 24,
};


static struct omap_dss_device *mvblx_dss_devices[] = {
	&mvblx_dvi_device,
};

static struct omap_dss_board_info mvblx_dss_data = {
	.num_devices = ARRAY_SIZE(mvblx_dss_devices),
	.devices = mvblx_dss_devices,
	.default_device = &mvblx_dvi_device,
};

static struct platform_device mvblx_dss_device = {
	.name          = "omapdss",
	.id            = -1,
	.dev            = {
		.platform_data = &mvblx_dss_data,
	},
};

static struct regulator_consumer_supply mvblx_vdvi_supply =
	REGULATOR_SUPPLY("vdds_dsi", "omapdss");

static void __init mvblx_display_init(void)
{
}


static struct omap2_hsmmc_info mmc[] = {
	{
		.name		= "internal",
		.mmc		= 1,
		.caps		= MMC_CAP_4_BIT_DATA,
		.gpio_wp	= -EINVAL,
		.gpio_cd	= -EINVAL,
		.ocr_mask	= MMC_VDD_30_31, /* VDD voltage 3.0 ~ 3.1 */
	},
	{
		.name		= "external",
		.mmc		= 2,
		.caps		= MMC_CAP_4_BIT_DATA,
		.gpio_wp	= -EINVAL,
		.gpio_cd	= -EINVAL,
		.ext_clock	= 0,
		.ocr_mask	= MMC_VDD_30_31, /* VDD voltage 3.0 ~ 3.1 */
	},
	{}	/* Terminator */
};

#ifdef CONFIG_REGULATOR_FIXED_VOLTAGE
static struct regulator_consumer_supply mvblx_vmmc1_supply =
	REGULATOR_SUPPLY("vmmc", "mmci-omap-hs.0");

static struct regulator_consumer_supply mvblx_vmmc2_supply =
	REGULATOR_SUPPLY("vmmc", "mmci-omap-hs.1");
#endif

static int mvblx_twl_gpio_setup(struct device *dev,
		unsigned gpio, unsigned ngpio)
{
#define GPIO_MMC0_CD 1
#define GPIO_MMC1_CD 0
#define GPIO_USB_OC  2

	/* card detects (input/IRQ) */
	mmc[0].gpio_cd = gpio + GPIO_MMC0_CD;
	mmc[1].gpio_cd = gpio + GPIO_MMC1_CD;
	omap2_hsmmc_init(mmc);

	/* USB overcurrent input */
	gpio_request(gpio + GPIO_USB_OC, "USB_nOC");
	gpio_direction_input(gpio + GPIO_USB_OC);

	/* USB power on/off control */
	gpio_request(gpio + TWL4030_GPIO_MAX, "nEN_USB_PWR");
	gpio_direction_output(gpio + TWL4030_GPIO_MAX, 0);

	return 0;
}

static struct twl4030_gpio_platform_data mvblx_gpio_data = {
	.gpio_base	= OMAP_MAX_GPIO_LINES,
	.irq_base	= TWL4030_GPIO_IRQ_BASE,
	.irq_end	= TWL4030_GPIO_IRQ_END,
	.use_leds	= false,
	.pullups	= 0, 
	.pulldowns	= BIT(6) | BIT(7) | BIT(13) | BIT(15),
	.setup		= mvblx_twl_gpio_setup,
};

/* VIO for digital video outputs */
static struct regulator_init_data mvblx_vio = {
	.constraints = {
		.name			= "VIO",
		.min_uV			= 1800000,
		.max_uV			= 1800000,
		.valid_modes_mask	= REGULATOR_MODE_NORMAL,
		.valid_ops_mask		= 0,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &mvblx_vdvi_supply,
};

#ifdef CONFIG_REGULATOR_FIXED_VOLTAGE
/* fixed voltage regulator @ 3.0V for MMC1 = internal */
static struct regulator_init_data mvblx_vmmc1 = {
	.constraints = {
		.valid_modes_mask	= REGULATOR_MODE_NORMAL,
		.always_on		= 1,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &mvblx_vmmc1_supply,
};

static struct fixed_voltage_config mvblx_fix_mmc1 = {
	.supply_name		= "vmmcfix1",
	.microvolts		= 3000000,
	.gpio			= -EINVAL,
	.enabled_at_boot	= 1,
	.init_data		= &mvblx_vmmc1,
};

static struct platform_device mvblx_vmmc1_device = {
	.name		= "reg-fixed-voltage",
	.id		= 0,
	.dev = {
		.platform_data  = &mvblx_fix_mmc1,
	},
};

/* fixed voltage regulator @ 3.0V for MMC2 = external */
static struct regulator_init_data mvblx_vmmc2 = {
	.constraints = {
		.valid_modes_mask	= REGULATOR_MODE_NORMAL,
		.always_on		= 1,
	},
	.num_consumer_supplies	= 1,
	.consumer_supplies	= &mvblx_vmmc2_supply,
};

static struct fixed_voltage_config mvblx_fix_mmc2 = {
	.supply_name		= "vmmcfix2",
	.microvolts		= 3000000,
	.gpio			= -EINVAL,
	.enabled_at_boot	= 1,
	.init_data		= &mvblx_vmmc2,
};

static struct platform_device mvblx_vmmc2_device = {
	.name		= "reg-fixed-voltage",
	.id		= 1,
	.dev = {
		.platform_data  = &mvblx_fix_mmc2,
	},
};
#endif /* CONFIG_REGULATOR_FIXED_VOLTAGE */

static struct twl4030_usb_data mvblx_usb_data = {
	.usb_mode	= T2_USB_MODE_ULPI,
};

static struct twl4030_codec_audio_data mvblx_audio_data = {
	.audio_mclk = 26000000,
};

static struct twl4030_codec_data mvblx_codec_data = {
	.audio_mclk = 26000000,
	.audio = &mvblx_audio_data,
};

static int board_keymap[] = {
	KEY(0, 0, KEY_F1),
	KEY(0, 1, KEY_F2),
	KEY(0, 2, KEY_ENTER)
};

static struct matrix_keymap_data board_map_data = {
	.keymap		= board_keymap,
	.keymap_size	= ARRAY_SIZE(board_keymap),
};

static struct twl4030_keypad_data mvblx_kp_data = {
	.keymap_data	= &board_map_data,
	.rows		= 3,
	.cols		= 1,
	.rep		= 1,
};


static struct twl4030_platform_data mvblx_twldata = {
	.irq_base	= TWL4030_IRQ_BASE,
	.irq_end	= TWL4030_IRQ_END,

	/* platform_data for children goes here */
	.keypad		= &mvblx_kp_data,
	.usb		= &mvblx_usb_data,
	.gpio		= &mvblx_gpio_data,
	.codec		= &mvblx_codec_data,
	.vio		= &mvblx_vio,
};

static struct i2c_board_info __initdata mvblx_i2c_boardinfo_bus1[] = {
	{
		I2C_BOARD_INFO("twl4030", 0x48),
		.flags = I2C_CLIENT_WAKE,
		.irq = INT_34XX_SYS_NIRQ,
		.platform_data = &mvblx_twldata,
	},
};

static struct i2c_board_info __initdata mvblx_i2c_boardinfo_bus2[] = {
	{ I2C_BOARD_INFO("eeprom", (0x6E>>1)) },
	{ I2C_BOARD_INFO("eeprom", (0xA0>>1)) },
	{ I2C_BOARD_INFO("ds1307", 0x68) },
};


static struct __attribute__ ((__packed__)) sys_eeprom {
	u8 res1[32];		/* 0x01 - 0x1F system info */
	u8 mac[ETH_ALEN];	/* 0x20 - 0x25 MAC address  */
	u8 res2[10];		/* 0x26 - 0x2f reserved */
	u32 crc;		/* 0x30 - 0x33 CRC32 checksum */
} e;

static u32 ones_complement_crc32 (u32 crc, unsigned char const *p, size_t len)
{
     return crc32(crc ^ 0xffffffffL, p, len) ^ 0xffffffffL;
}

static void mvblx_get_mac_addr(struct memory_accessor *mem_acc, void *context)
{
	unsigned char *mac_addr = mvblx_smsc911x_config.mac;

	if (mem_acc->read(mem_acc, (unsigned char*)&e, 0, sizeof(e)) == sizeof(e)) {
		u32 crc, crc_offset = offsetof(struct sys_eeprom, crc);

		crc = ones_complement_crc32(0, (void *)&e, crc_offset);
		if (crc == be32_to_cpu(e.crc)) {
			memcpy(mac_addr, e.mac, ETH_ALEN);
			printk("Read MAC addr from EEPROM: %pM\n", mac_addr);
		} else {
			printk("System EEPROM CRC error\n");
		}
	}
}

static struct at24_platform_data mcp98242 = {
	.byte_len	= 2 * SZ_1K / 8,
	.page_size	= 16,
	.flags		= AT24_FLAG_READONLY | AT24_FLAG_IRUGO,
	.setup		= mvblx_get_mac_addr,
};

static struct at24_platform_data m24128 = {
	.byte_len	= SZ_128K / 8,
	.page_size	= 64,
	.flags		= AT24_FLAG_ADDR16,
};

static struct i2c_board_info __initdata mvblx_i2c_boardinfo_bus3[] = {
	/* i2c devices on the sensor-PCB will be instantiated by mvi2c.ko */
	/* The following EEPROMs are on the main board */
	{	/* System EEPROM for serial number, MAC etc. */
		I2C_BOARD_INFO("at24", (0xA0>>1)),
		.platform_data = &mcp98242,
	},
	{	/* Additional EEPROM */
		I2C_BOARD_INFO("at24", (0xA2>>1)),
		.platform_data = &m24128,
	},
};

static int __init mvblx_i2c_init(void)
{
	omap_register_i2c_bus(1, 2600, mvblx_i2c_boardinfo_bus1,
			ARRAY_SIZE(mvblx_i2c_boardinfo_bus1));
	/* Bus 2 is attached to the VIDEO port where devices like the pico DLP
	 * projector don't work reliably with 400kHz */
	omap_register_i2c_bus(2, 100,  mvblx_i2c_boardinfo_bus2,
			ARRAY_SIZE(mvblx_i2c_boardinfo_bus2));
	/* Bus 3 for camera sensors */
	omap_register_i2c_bus(3, 400,  mvblx_i2c_boardinfo_bus3,
			ARRAY_SIZE(mvblx_i2c_boardinfo_bus3));
#if 0
	/* Bus 4 for 2nd interface to twl4030 */
	omap_register_i2c_bus(4, 2600, NULL, 0);
#endif
	return 0;
}

static void mvblx_set_iva2_freq(void)
{
	struct clk *iva2_ck, *dpll2_ck;
	const char *iva2_ck_name = "iva2_ck";
	const char *dpll2_ck_name = "dpll2_ck";
	unsigned long iva2_ck_rate;

	dpll2_ck = clk_get(NULL, dpll2_ck_name);
	if (WARN(IS_ERR(dpll2_ck), "clock: failed to get %s.\n", dpll2_ck_name))
		return;

	iva2_ck = clk_get(NULL, iva2_ck_name);
	if (WARN(IS_ERR(iva2_ck), "clock: failed to get %s.\n", iva2_ck_name))
		return;

	/* REVISIT: use OPP table to get these frequencies */
	if (iva2rate > 0) {
		if (iva2rate > 800000000L) {
			iva2rate = 800000000L;
		}
		iva2_ck_rate = iva2rate;
		if (clk_set_rate(dpll2_ck, iva2_ck_rate))
			printk(KERN_ERR "*** Unable to set IVA2 rate to %ld MHz\n", (iva2_ck_rate / 1000000));
		clk_enable(iva2_ck);
	}

	iva2_ck_rate = clk_get_rate(iva2_ck);
	pr_info("mvblx IVA2 clocking rate: %ld MHz\n", (iva2_ck_rate / 1000000));
}

/* 1GHz-Patch from Max Galemin: workaround until full support is in the kernel */
#define SET_VDD1_VSEL_1_35_V    0x3C
int mvblx_set_opp1g_voltage(void)
{
	if (cpu_is_omap3630()) {
		/* See TPS65950 TRM (SWCU050F) "5.4.10.1.1 Direct Control" for details. */
		printk(KERN_DEBUG "setting up OPP for 1 GHz (1.35V)\n");
		twl_i2c_write_u8(TWL4030_MODULE_PM_RECEIVER,
							SET_VDD1_VSEL_1_35_V,
							TWL4030_VDD1_VSEL);
		mvblx_set_iva2_freq();
	}
	return 0;
}

/* Matrix Vision FPGA */
#if defined(CONFIG_MEDIA_CONTROLLER)
static struct omap2_mcspi_device_config mvfpga_mcspi_config = {
	.turbo_mode	= 0,
	.single_channel	= 1,
};

/* there is one spi_board_info element for each device on an SPI bus */
struct spi_board_info mvblx_spi_board_info[] = {
	[0] = {
#if USE_SPIDEV_DEBUG
		.modalias        = "spidev",
#else
		.modalias        = "mvfw",
#endif
		/* JOE don't know how to pass mvfpga_platform_data to an
		 * externally-built module. Don't have anything to send anyway.
		 */
		.platform_data		= NULL,
		.bus_num         = 2,
		.chip_select     = 0,
		/*.max_speed_hz    = 1500000, */
		.max_speed_hz    = 24000000,
		.controller_data = &mvfpga_mcspi_config,
	},
};

/* This doesn't appear in the mvblx_spi_board_info array because
 * it will be registered when requested by the omap3isp module.
 * This is a dummy SPI device, it shouldn't make any SPI accesses
 * itself.
 */
struct spi_board_info mvblx_blxcam_board_info = {
	.modalias = "blxcam",
	/* since this is a dummy, we just need to be sure we get a bus_num
	 * and chip_select which aren't already taken.
	 */
	.bus_num  = 2,
	.chip_select = 1,
};

static struct isp_subdev_bus_board_info mvblx_camera_bus_boardinfos[] = {
	{
		.bustype = ISP_SUBDEV_SPI_BUS,
		.businfo = { .spi_info = &mvblx_blxcam_board_info },
	},
	{},
};

static struct isp_v4l2_subdevs_group mvblx_camera_subdevs[] = {
	{
		.subdevs = mvblx_camera_bus_boardinfos,
		.interface = ISP_INTERFACE_PARALLEL,
		.bus = { .parallel = {
			.data_lane_shift = 0,
			.clk_pol = 0,
			.bridge = ISPCTRL_PAR_BRIDGE_DISABLE,
		} },
	},
	{},
};

static struct isp_platform_data mvblx_isp_platform_data = {
	.subdevs = mvblx_camera_subdevs,
};
#endif /* CONFIG_MEDIA_CONTROLLER */

#if defined(CONFIG_MEDIA_CONTROLLER)
static int __init mvblx_camera_init(void)
{
	return omap3_init_camera(&mvblx_isp_platform_data);
}
#else
static int __init mvblx_camera_init(void) { return 0; }
#endif /* CONFIG_MEDIA_CONTROLLER */

static void __init mvblx_init_irq(void)
{
	omap2_init_common_infrastructure();
	omap2_init_common_devices(mt46h32m32lf6_sdrc_params,
			     mt46h32m32lf6_sdrc_params);
	omap_init_irq();
#ifdef CONFIG_OMAP_32K_TIMER
	omap2_gp_clockevent_set_gptimer(12);
#endif
}


static struct w1_gpio_platform_data mvblx_w1_gpio_pdata = {
	.pin			= MVBLX_GIPO_ONE_WIRE,
	.is_open_drain	= 0,		/* 0 = configure as an input */
};

static struct platform_device mvblx_w1_device = {
	.name				= "w1-gpio",
	.id					= -1,
	.dev.platform_data	= &mvblx_w1_gpio_pdata,
};

/* Use gpio_led to pass info although our LEDs are not
 * driven by GPIOs
 */
#define MVBLX_LED(__name, __default) \
{ \
	.name = "mvblx:" #__name, \
	.default_state = LEDS_GPIO_DEFSTATE_##__default, \
}
static struct gpio_led mvblx_leds[] = {
	MVBLX_LED(red:power,	OFF),
	MVBLX_LED(green:power,	ON),
	MVBLX_LED(red:usr1,	OFF),
	MVBLX_LED(green:usr1,	OFF),
	MVBLX_LED(red:usr2,	OFF),
	MVBLX_LED(green:usr2,	OFF),
	MVBLX_LED(red:usr3,	OFF),
	MVBLX_LED(green:usr3,	OFF),
};

static struct gpio_led_platform_data mvblx_led_info = {
	.leds		= mvblx_leds,
	.num_leds	= ARRAY_SIZE(mvblx_leds),
};

static struct platform_device mvblx_led_device = {
	.name	= "leds-mvblx",
	.id	= -1,
	.dev	= {
		.platform_data	= &mvblx_led_info,
	},
};

static struct platform_device *mvblx_devices[] __initdata = {
	&mvblx_dss_device,
#ifdef CONFIG_REGULATOR_FIXED_VOLTAGE
	&mvblx_vmmc1_device,
	&mvblx_vmmc2_device,
#endif
	&mvblx_w1_device,
	&mvblx_led_device,
};

static struct ehci_hcd_omap_platform_data ehci_pdata __initconst = {

	.port_mode[0] = EHCI_HCD_OMAP_MODE_PHY,

	/* port_mode[1] was trying to get same pins as MCSPI2
	 * in arch/arm/mach-omap2/usb-ehci.c
	 */
	.port_mode[1] = EHCI_HCD_OMAP_MODE_UNKNOWN,
	/*.port_mode[1] = EHCI_HCD_OMAP_MODE_PHY, */
	.port_mode[2] = EHCI_HCD_OMAP_MODE_UNKNOWN,

	.phy_reset  = true,
	.reset_gpio_port[0]  = MVBLX_GPIO_USBH_NRESET,
	.reset_gpio_port[1]  = -EINVAL,
	.reset_gpio_port[2]  = -EINVAL
};

#ifdef CONFIG_OMAP_MUX
static struct omap_board_mux board_mux[] __initdata = {
	{ .reg_offset = OMAP_MUX_TERMINATOR },
};
#else
#define board_mux	NULL
#endif

static struct omap_musb_board_data musb_board_data = {
	.interface_type		= MUSB_INTERFACE_ULPI,
#if defined(CONFIG_USB_MUSB_OTG)
	.mode			= MUSB_OTG,
#elif defined(CONFIG_USB_GADGET_MUSB_HDRC)
	.mode			= MUSB_PERIPHERAL,
#else
	.mode			= MUSB_HOST,
#endif
	.power			= 100,
};

static void __init mvblx_init(void)
{
	struct omap_board_data bdata;
	int i;

	omap3_mux_init(board_mux, OMAP_PACKAGE_CUS);
	mvblx_init_rev();
	mvblx_i2c_init();
	platform_add_devices(mvblx_devices,
			ARRAY_SIZE(mvblx_devices));

	for (i=0; i<3; i++) {
		bdata.id = i;
		bdata.flags = 0;
		bdata.pads = NULL;
		bdata.pads_cnt = 0;
		omap_serial_init_port(&bdata);
	}

	usb_musb_init(&musb_board_data);
	usb_ehci_init(&ehci_pdata);

#if defined(CONFIG_SMSC911X) || defined(CONFIG_SMSC911X_MODULE)
	mvblx_init_smsc911x();
#endif /* SMSC911X */

	/* Ensure SDRC pins are mux'd for self-refresh */
	omap_mux_init_signal("sdrc_cke0", OMAP_PIN_OUTPUT);
	/* we don't yet have a second memory bank but do this anyway */
	omap_mux_init_signal("sdrc_cke1", OMAP_PIN_OUTPUT);

#if defined(CONFIG_MEDIA_CONTROLLER)
	spi_register_board_info(mvblx_spi_board_info,
			ARRAY_SIZE(mvblx_spi_board_info));
#endif
	mvblx_display_init();
	mvblx_camera_init();
}

MACHINE_START(MVBLX, "MATRIX VISION mvBlueLYNX-X")
	.boot_params	= 0x80000100,
	.map_io		= omap3_map_io,
	.reserve	= omap_reserve,
	.init_irq	= mvblx_init_irq,
	.init_machine	= mvblx_init,
	.timer		= &omap_timer,
MACHINE_END
